package org.eclipse.cosmos.rm.validation.internal.databuilders;

import java.util.Map;

import junit.framework.Test;

import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.cosmos.rm.validation.internal.common.UtilityTester;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;

/**
 * Generated code for the test suite <b>TestTypeInheritanceDataBuilderImpl</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/junits/TestTypeInheritanceDataBuilderImpl.testsuite</i>.
 */
public class TestTypeInheritanceDataBuilderImpl extends HyadesTestCase
{
	/**
	 * Constructor for TestTypeInheritanceDataBuilderImpl.
	 * @param name
	 */
	public TestTypeInheritanceDataBuilderImpl(String name)
	{
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestTypeInheritanceDataBuilderImpl</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testTypeInheritanceDataBuilderImpl = new HyadesTestSuite("TestTypeInheritanceDataBuilderImpl");
		testTypeInheritanceDataBuilderImpl.setArbiter(DefaultTestArbiter.INSTANCE).setId("EAE0EB6460661010069753B07BFB11DB");
		testTypeInheritanceDataBuilderImpl.addTest(new TestTypeInheritanceDataBuilderImpl("testInheritanceMapGeneration").setId("EAE0EB646066101008CC5C207BFB11DB").setTestInvocationId("EAE0EB64606610101378C0507BFB11DB"));
		testTypeInheritanceDataBuilderImpl.addTest(new TestTypeInheritanceDataBuilderImpl("testBadDerivation").setId("EAE0EB6460661010093B3DD07E5C11DB").setTestInvocationId("EAE0EB646066101015B3AF707E5C11DB"));
		testTypeInheritanceDataBuilderImpl.addTest(new TestTypeInheritanceDataBuilderImpl("testValidDerivation").setId("EAE0EB6460661010069BEA807F0F11DB").setTestInvocationId("EAE0EB64606610102BC020107F0F11DB"));
		return testTypeInheritanceDataBuilderImpl;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	/**
	 * testInheritanceMapGeneration
	 * @throws Exception
	 */
	public void testInheritanceMapGeneration() throws Exception
	{
		TypeInheritanceDataBuilderImpl inheritanceMapBuilder = initialize ("test-resources/invalid-cycle.xml");
		
		Map inheritanceMap = (Map)inheritanceMapBuilder.getDataStructure();
		final String NAME_SPACE = "urn:university";
		final String REQUIRE_COURSE_TYPE = "RequiredCourseRefType";
		final String OPTIONAL_COURSE_TYPE = "OptionalCourseRefType";
		
		assertEquals("The " + REQUIRE_COURSE_TYPE + " is expected to be an extension of " + OPTIONAL_COURSE_TYPE, NAME_SPACE+OPTIONAL_COURSE_TYPE, (String)inheritanceMap.get(NAME_SPACE+REQUIRE_COURSE_TYPE));
		assertEquals("The " + OPTIONAL_COURSE_TYPE + " is expected to be an extesion of " + ISMLConstants.REF_TYPE_VALUE, ISMLConstants.SML_URI+ISMLConstants.REF_TYPE_VALUE, (String)inheritanceMap.get(NAME_SPACE+OPTIONAL_COURSE_TYPE));		
	}

	/**
	 * testBadDerivation
	 * Tests for bad derivation:
	 * R2 extends R1, where R1(acyclic=true) and R2(acyclic=false)
	 * @throws Exception
	 */
	public void testBadDerivation() throws Exception
	{
		TypeInheritanceDataBuilderImpl inheritanceMapBuilder = initialize ("test-resources/invalid-derivation.xml");
		
		assertTrue("The inheritance map was expected to contain at least one element", ((Map)inheritanceMapBuilder.getDataStructure()).size() > 0);
		assertFalse("The map was expected to be invalid", inheritanceMapBuilder.isStructureValid());
		assertTrue("The builder was expected to have an associated error message", inheritanceMapBuilder.getErrorMessage().length() > 0);
	}

	/**
	 * testValidDerivation 
	 * 
	 * A model is valid if it has a reference type R1 with sml:acyclic=”false” 
	 * and another reference type R2 derived from R1 such that sml:acyclic=”true”.
	 * @throws Exception
	 */
	public void testValidDerivation() throws Exception
	{
		TypeInheritanceDataBuilderImpl inheritanceMapBuilder = initialize ("test-resources/valid-derivation.xml");
		
		final int EXPECTED_MAP_SIZE = 2;
		
		assertEquals("The inheritance map was expected to contain " + EXPECTED_MAP_SIZE + " elements", EXPECTED_MAP_SIZE, ((Map)inheritanceMapBuilder.getDataStructure()).size());
		assertTrue("The map was expected to be valid", inheritanceMapBuilder.isStructureValid());
		assertNull("The builder was expected not to have an associated error message", inheritanceMapBuilder.getErrorMessage());
	}

	
	private TypeInheritanceDataBuilderImpl initialize(String fileName) throws Exception
	{
		TypeInheritanceDataBuilderImpl inheritanceMapBuilder = new TypeInheritanceDataBuilderImpl();		
		UtilityTester.parseDocument(this.getClass().getClassLoader().getResourceAsStream(fileName), inheritanceMapBuilder);
		return inheritanceMapBuilder;
	}
}
