package org.eclipse.cosmos.rm.validation.internal.reference;

import java.net.URISyntaxException;

import junit.framework.Test;

import org.eclipse.cosmos.rm.validation.internal.common.UtilityTester;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;

/**
 * Generated code for the test suite <b>TestURIReference</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/junits/TestURIReference.testsuite</i>.
 */
public class TestURIReference extends HyadesTestCase
{
	/**
	 * Constructor for TestURIReference.
	 * @param name
	 */
	public TestURIReference(String name)
	{
		super(name);
	}
   
	/**
	 * Returns the JUnit test suite that implements the <b>TestURIReference</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testURIReference = new HyadesTestSuite("TestURIReference");
		testURIReference.setArbiter(DefaultTestArbiter.INSTANCE).setId("C7D7CF520CB65742EC2C97A0847F11DB");
		testURIReference.addTest(new TestURIReference("testTransformAbsolute").setId("C7D7CF520CB65742F2DC79D0847F11DB").setTestInvocationId("C7D7CF520CB657423433EC60848011DB"));
		testURIReference.addTest(new TestURIReference("testTransformRelative").setId("C7D7CF520CB65742FB586970847F11DB").setTestInvocationId("C7D7CF520CB65742343B1850848011DB"));
		testURIReference.addTest(new TestURIReference("testTransformRelativeMerge1").setId("C7D7CF520CB657421C4E1080848011DB").setTestInvocationId("C7D7CF520CB6574292A86D70848511DB"));
		testURIReference.addTest(new TestURIReference("testTransformRelativeMerge2").setId("C7D7CF520CB6574203D48DE0848511DB").setTestInvocationId("C7D7CF520CB6574292AADE70848511DB"));
		testURIReference.addTest(new TestURIReference("testTransformRelativeMerge3").setId("C7D7CF520CB65742062FBBF0848511DB").setTestInvocationId("C7D7CF520CB6574292AF9960848511DB"));
		testURIReference.addTest(new TestURIReference("testTransformWithDots1").setId("C7D7CF520CB65742016AFA30848011DB").setTestInvocationId("C7D7CF520CB65742BC109C00848A11DB"));
		testURIReference.addTest(new TestURIReference("testTransformWithDots2").setId("C7D7CF520CB657426AC76F90848A11DB").setTestInvocationId("C7D7CF520CB65742BC130D00848A11DB"));
		return testURIReference;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	/**
	 * testTransformAbsolute
	 * Base = "http://interop.serviceml.org"
	 * URI = "http://absolute.org/mydocument"
	 * @throws Exception
	 */
	public void testTransformAbsolute() throws Exception
	{
		testMerge (	"http://interop.serviceml.org", 
					"http://absolute.org/mydocument",
					"http://absolute.org/mydocument");
	}

	/**
	 * testTransformRelative
	 * Base = "http://interop.serviceml.org"
	 * URI = "#xmlns(u=urn:university)xpointer(/u:University/u:Courses/u:Course[u:Name=’MAT200’])"
	 * @throws Exception
	 */
	public void testTransformRelative() throws Exception
	{
		testMerge (	"http://interop.serviceml.org", 
					"#xmlns(u=urn:university)xpointer(/u:University/u:Courses/u:Course[u:Name=’MAT200’]",
					"http://interop.serviceml.org#xmlns(u=urn:university)xpointer(/u:University/u:Courses/u:Course[u:Name=’MAT200’]");
	}

	/**
	 * testTransformRelativeMerge1 Base = "http://interop.serviceml.org/segment1/segment2" 
	 * URI = "segment3/segment4#somequery"
	 * @throws Exception
	 */
	public void testTransformRelativeMerge1() throws Exception
	{
		testMerge (	"http://interop.serviceml.org/segment1/segment2", 
					"segment3/segment4#somequery",
					"http://interop.serviceml.org/segment1/segment3/segment4#somequery");
	}

	/**
	 * testTransformRelativeMerge2
	 * Base = "http://interop.serviceml.org/segment1/segment2/"
	 * URI = "segment3/segment4#somequery"
	 * @throws Exception
	 */
	public void testTransformRelativeMerge2() throws Exception
	{
		testMerge (	"http://interop.serviceml.org/segment1/segment2/", 
					"segment3/segment4#somequery",
					"http://interop.serviceml.org/segment1/segment2/segment3/segment4#somequery");
	}

	/**
	 * testTransformRelativeMerge3
	 * Base = "http://interop.serviceml.org/segment1/segment2/"
	 * URI = "/segment3/segment4#somequery"
	 * @throws Exception
	 */
	public void testTransformRelativeMerge3() throws Exception
	{
		testMerge (	"http://interop.serviceml.org/segment1/segment2/", 
					"/segment3/segment4#somequery",
					"http://interop.serviceml.org/segment3/segment4#somequery");
	}


	/**
	 * testTransformWithDots1 Base = "http://interop.serviceml.org/segment1/segment2/" 
	 * URI = "../segment3#somequery"
	 * @throws Exception
	 */
	public void testTransformWithDots1() throws Exception
	{
		testMerge (	"http://interop.serviceml.org/segment1/segment2/", 
					"../segment3#somequery",
					"http://interop.serviceml.org/segment1/segment3#somequery");
	}

	/**
	 * testTransformWithDots2 Base = "http://interop.serviceml.org/segment1/segment2/"  URI = "./segment3#somequery"
	 * @throws Exception
	 */
	public void testTransformWithDots2() throws Exception
	{
		testMerge (	"http://interop.serviceml.org/segment1/segment2/", 
				"./segment3#somequery",
				"http://interop.serviceml.org/segment1/segment2/segment3#somequery");
	}
	
	private void testMerge(String base, String reference, final String EXPECTED) throws URISyntaxException
	{
		URIReference uriReference = new URIReference(reference);
		UtilityTester.registerIdentityStructure(null, null, base);		
		assertEquals("The merged URI is not what was expected", EXPECTED, uriReference.getDocumentReference());
	}
}
