package org.eclipse.cosmos.rm.validation.internal.reference;

import java.util.Hashtable;
import java.util.List;

import junit.framework.Test;

import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;

/**
 * Generated code for the test suite <b>TestXPointer</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/junits/TestXPointer.testsuite</i>.
 */
public class TestXPointer extends HyadesTestCase
{
	/**
	 * Constructor for TestXPointer.
	 * @param name
	 */
	public TestXPointer(String name)
	{
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestXPointer</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testXPointer = new HyadesTestSuite("TestXPointer");
		testXPointer.setArbiter(DefaultTestArbiter.INSTANCE).setId("F047EAA18640A2EDC8A94C2089F611DB");
		testXPointer.addTest(new TestXPointer("testCompileFullExpression").setId("F047EAA18640A2EDCB5F1D5089F611DB").setTestInvocationId("F047EAA18640A2ED2756F7E089F711DB"));
		testXPointer.addTest(new TestXPointer("testCompileChildSequence").setId("F047EAA18640A2EDD870257089F611DB").setTestInvocationId("F047EAA18640A2ED27606DC089F711DB"));
		testXPointer.addTest(new TestXPointer("testCompileBareName").setId("F047EAA18640A2EDE11E219089F611DB").setTestInvocationId("F047EAA18640A2ED276528B089F711DB"));
		testXPointer.addTest(new TestXPointer("testBadChildSequence").setId("F047EAA18640A2EDE9D5BAA089F611DB").setTestInvocationId("F047EAA18640A2ED276A0AB089F711DB"));
		testXPointer.addTest(new TestXPointer("testBadFullExpression").setId("F047EAA18640A2EDF3F2FE3089F611DB").setTestInvocationId("F047EAA18640A2ED276C54A089F711DB"));
		testXPointer.addTest(new TestXPointer("testEscapedCharacters").setId("F047EAA18640A2EDF868F5F089F611DB").setTestInvocationId("F047EAA18640A2ED27710F9089F711DB"));
		return testXPointer;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	/**
	 * testCompileFullExpression
	 * @throws Exception
	 */
	public void testCompileFullExpression() throws Exception
	{
		final String XPOINTER_EXPRESSION = "xmlns(u=urn:university) xpointer (/u:Courses/u:Course[u:Name=’Physics101’]) xmlns(u=urn:university)xpointer(/u:Courses/u:Course[u:Name=’Physics201’])unknown(unknown)";
		XPointerExpression xpointerExpression = XPointer.compile(XPOINTER_EXPRESSION);
		List schemes = xpointerExpression.getScheme();
		
		assertEquals("The expected type of the expression is full", XPointerExpression.FULL_XPOINTER_TYPE, xpointerExpression.getType());
		assertNotNull("The schemes of the expression is not expected to be null", schemes);
		assertEquals("The expected size of the scheme is 5", 5, schemes.size());
		
		int xpointerCount = 0, xmlnsCount = 0;
		Hashtable expressions = new Hashtable();
		for (int i = 0, schemeCount = schemes.size(); i < schemeCount; i++)
		{
			IXScheme scheme = (IXScheme)schemes.get(i);
			if (XPointerScheme.SCHEME.equals(scheme.getType()))
			{
				xpointerCount++;
			}
			else if (XmlnsScheme.SCHEME.equals(scheme.getType()))
			{
				xmlnsCount++;
			}
			
			String expression = scheme.getExpression();
			if (expression != null)
				expressions.put(expression, "");
		}
		
		assertEquals("The expected number of xpointer schemes is 2", 2, xpointerCount);
		assertEquals("The expected number of xmlns schemes is 2", 2, xmlnsCount);
		
		assertNotNull("The expression 'u=urn:university' was not found", expressions.get("u=urn:university"));
		assertNotNull("The expression '/u:Courses/u:Course[u:Name=’Physics101’]' was not found", expressions.get("/u:Courses/u:Course[u:Name=’Physics101’]"));
		assertNotNull("The expression '/u:Courses/u:Course[u:Name=’Physics101’]' was not found", expressions.get("/u:Courses/u:Course[u:Name=’Physics201’]"));
	}

	/**
	 * testCompileChildSequence
	 * @throws Exception
	 */
	public void testCompileChildSequence() throws Exception
	{
		final String BARE_NAME_CHILD_SEQ = "intro/14/3";
		XPointerExpression xpointerExpression = XPointer.compile(BARE_NAME_CHILD_SEQ);
		int[] childSeq = xpointerExpression.getChildSequence();
		
		assertEquals("The expected type of the expression is child seq", XPointerExpression.CHILD_SEQUENCE_TYPE, xpointerExpression.getType());
		assertEquals("The bare name of the expression is expected to be 'intro'", "intro", xpointerExpression.getBareName());
		assertEquals("The child sequence length is expected to be 2", 2, childSeq.length);
		checkChildSequence(new int[]{14, 3}, childSeq);
		
		final String CHILD_SEQ = "/1/2/5/14/3";
		xpointerExpression = XPointer.compile(CHILD_SEQ);
		childSeq = xpointerExpression.getChildSequence();
		checkChildSequence(new int[]{1, 2, 5, 14, 3}, childSeq);		
	}

	private void checkChildSequence(int[] expected, int[] actual)
	{
		for (int i = 0; i < actual.length; i++)
			assertEquals("The child sequence with index: " + i + " is not what was expected", expected[i], actual[i] + 1);		
	}

	/**
	 * testCompileBareName
	 * @throws Exception
	 */
	public void testCompileBareName() throws Exception
	{
		final String BARE_NAME = "intro";
		XPointerExpression xpointerExpression = XPointer.compile(BARE_NAME);
		
		assertEquals("The expected type of the expression is bare name", XPointerExpression.BARE_NAME_TYPE, xpointerExpression.getType());
		assertEquals("The bare name of the expression is expected to be 'intro'", "intro", xpointerExpression.getBareName());
	}

	/**
	 * testBadChildSequence
	 * @throws Exception
	 */
	public void testBadChildSequence() 
	{
		final String BAD_SEQ_EXPRESSION = "intro/14/3/bad-sector/5";
		try
		{
			XPointer.compile(BAD_SEQ_EXPRESSION);
			fail("The compilation of the method was expected to throw an exception");
		} 
		catch (XPointerSyntaxException e)
		{
			assertEquals("The token, bad-sector, in the child sequence XPointer expression, intro/14/3/bad-sector/5, was expected to be a number", e.getMessage());
		}
	}

	/**
	 * testBadFullExpression
	 * @throws Exception
	 */
	public void testBadFullExpression()
	{
		final String BAD_FULL_EXPRESSION = "xmlns(u=urn:university) xpointer) (/u:Courses/u:Course[u:Name=’Physics101’]) xmlns(u=urn:university)xpointer(/u:Courses/u:Course[u:Name=’Physics201’])unknown(unknown)";
		
		try
		{
			XPointer.compile(BAD_FULL_EXPRESSION);
			fail("An exception was expecetd to be thrown");
		} 
		catch (XPointerSyntaxException e)
		{
			assertEquals("An unbalanced bracket was detected when reading the XPointer expression: xmlns(u=urn:university) xpointer) (/u:Courses/u:Course[u:Name=’Physics101’]) xmlns(u=urn:university)xpointer(/u:Courses/u:Course[u:Name=’Physics201’])unknown(unknown)",
					e.getMessage());
		}
		
		
		final String BAD_FULL_EXPRESSION2 = "xmlns(u=urn:university) xpointer(unfinished expression";
		
		try
		{
			XPointer.compile(BAD_FULL_EXPRESSION2);
			fail("An exception was expecetd to be thrown");
		} 
		catch (XPointerSyntaxException e)
		{
			assertEquals("The scheme expression \"unfinished expression\" is not closed",
					e.getMessage());
		}
	}

	/**
	 * testEscapedCharacters
	 * @throws Exception
	 */
	public void testEscapedCharacters() throws Exception
	{
		final String FULL_EXPRESSION = "xmlns(something with bracket ^(part of expression^))";
		XPointerExpression expression = XPointer.compile(FULL_EXPRESSION);
		
		assertEquals("The number of schemes is expected to be 1" , 1, expression.getScheme().size());
		assertEquals("The expression is not what is expected", "something with bracket (part of expression)", ((IXScheme)expression.getScheme().get(0)).getExpression());
	}

}
