package org.eclipse.cosmos.rm.validation.internal.reference;

import junit.framework.Test;

import org.eclipse.cosmos.rm.validation.internal.common.UtilityTester;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DataBuilderRegistry;
import org.eclipse.cosmos.rm.validation.internal.databuilders.NamespaceContextBuilder;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;
import org.w3c.dom.Node;

/**
 * Generated code for the test suite <b>TestXPointerExpression</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/junits/TestXPointerExpression.testsuite</i>.
 */
public class TestXPointerExpression extends HyadesTestCase
{
	/**
	 * Constructor for TestXPointerExpression.
	 * @param name
	 */
	public TestXPointerExpression(String name)
	{
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestXPointerExpression</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testXPointerExpression = new HyadesTestSuite("TestXPointerExpression");
		testXPointerExpression.setArbiter(DefaultTestArbiter.INSTANCE).setId("F047EAA18640A2ED5455AB208A3211DB");
		testXPointerExpression.addTest(new TestXPointerExpression("testFullExpression").setId("F047EAA18640A2ED85528FE08A3211DB").setTestInvocationId("F047EAA18640A2EDA1B800708A3211DB"));
		testXPointerExpression.addTest(new TestXPointerExpression("testChildSequence").setId("F047EAA18640A2ED92F047A08A3211DB").setTestInvocationId("F047EAA18640A2EDA1BF05508A3211DB"));
		return testXPointerExpression;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	/**
	 * testFullExpression
	 * @throws Exception
	 */
	public void testFullExpression() throws Exception
	{
		DataBuilderRegistry.instance().registerDataStructureBuilder(NamespaceContextBuilder.ID, new NamespaceContextBuilder());
		XmlnsScheme xmlnsScheme = new XmlnsScheme();
		xmlnsScheme.setExpression("u=urn:university");
		
		XPointerScheme xpointerScheme = new XPointerScheme();
		xpointerScheme.setExpression("/u:Courses/u:Course[u:Name='Physics101']");
		
		XPointerExpression xpointerExpression = new XPointerExpression(XPointerExpression.FULL_XPOINTER_TYPE);
		xpointerExpression.addScheme(xmlnsScheme);
		xpointerExpression.addScheme(xpointerScheme);
			
		Object result = xpointerExpression.evaluate(UtilityTester.domParseDocument(this.getClass().getClassLoader().getResourceAsStream("test-resources/references/courses-with-ns.xml")));		
		assertTrue("The return type is expected to be Node", result instanceof Node);
		Node node = (Node)result;
		assertEquals("The node name is expected to be Course", "Course", node.getNodeName());		
	}

	/**
	 * testChildSequence
	 * @throws Exception
	 */
	public void testChildSequence() throws Exception
	{
		DataBuilderRegistry.instance().registerDataStructureBuilder(NamespaceContextBuilder.ID, new NamespaceContextBuilder());
		XPointerExpression xpointerExpression = new XPointerExpression(XPointerExpression.CHILD_SEQUENCE_TYPE);
		final int[] CHILD_SEQUENCE = new int[] {1, 2, 1, 1};
		
		for (int i = 0; i < CHILD_SEQUENCE.length; i++)
			xpointerExpression.addChildSequence(CHILD_SEQUENCE[i]);	
		
		
		Object result = xpointerExpression.evaluate(UtilityTester.domParseDocument(this.getClass().getClassLoader().getResourceAsStream("test-resources/references/courses-with-ns.xml")));
		assertTrue("The return type is expected to be Node", result instanceof Node);
		Node node = (Node)result;
		assertEquals("The node type is expected to be text", Node.TEXT_NODE, node.getNodeType());		
		assertEquals("The node value is expected to be Maths200", "Maths200", node.getNodeValue());
	}

}
