package org.eclipse.cosmos.rm.validation.internal.databuilders;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.cosmos.rm.validation.internal.SMLActivator;
import org.eclipse.cosmos.rm.validation.internal.artifacts.ElementTypeMap;
import org.eclipse.cosmos.rm.validation.internal.artifacts.TargetSchemaNode;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.cosmos.rm.validation.internal.util.ParserHelper;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class TargetSchemaBuilder extends AbstractDataBuilder implements
		IDataBuilder {

	public static final String ID = SMLActivator.PLUGIN_ID + ".TargetSchemaBuilder";
	protected Map dataStructure;
    protected boolean uriElementFound = false; 	
	
	public TargetSchemaBuilder(){		
		dataStructure = new HashMap();
	}

	public Object getDataStructure() {
		return dataStructure;
	}

	public void startSchema(String uri) {
	}
	
	public void endSchema(String uri) {
		IDataBuilder substitionElement  = DataBuilderRegistry.instance().getDataStructureBuilder(SubstitutionBuilder.ID);
		Map substitionElementDataStructure = ((Map)substitionElement.getDataStructure());
		
		//validate if target elements is defined on a derived type
		IDataBuilder refTypeElement = DataBuilderRegistry.instance().getDataStructureBuilder(RefTypeElementBuilder.ID);
		IDataBuilder elementTypeMapBuilder  = DataBuilderRegistry.instance().getDataStructureBuilder(ElementTypeMapDataBuilder.ID);
		ElementTypeMap elementTypeMapBuilderStructure = (ElementTypeMap)elementTypeMapBuilder.getDataStructure();
		IDataBuilder typeInheritance = DataBuilderRegistry.instance().getDataStructureBuilder(TypeInheritanceDataBuilderImpl.ID);
		Map refTypeElems = (Map)refTypeElement.getDataStructure();
		Map inheritance = (Map)typeInheritance.getDataStructure();
		
		Iterator iter = dataStructure.keySet().iterator();
		while (iter.hasNext()){
			Object elementName = iter.next();
			if (refTypeElems.containsKey(elementName))
				continue;
			//check inheritance tree
			Object targetRefTypeNS = elementTypeMapBuilderStructure.getType(uri, (String)elementName);
			String targetRefType = ParserHelper.removeNameSpace((String)targetRefTypeNS);
			Object parent = inheritance.get(targetRefType);
			boolean valid  = false;
			while (parent != null){
				if (refTypeElems.get(parent) != null){
					valid = true;
					break;
				}
				parent = inheritance.get(parent);				
			}
			if (!valid){				
				appendToErrorMessage("Target Element does not define sml:refType");
				return;
			}
		}
		
		
		//need to make sure that substitionElements are also considered
		iter = substitionElementDataStructure.keySet().iterator();
		while (iter.hasNext()){
			Object elementName = iter.next();
			String elementNameNonNS = ParserHelper.removeNameSpace((String)elementName);
			TargetSchemaNode targetSchemaNode = (TargetSchemaNode)dataStructure.get(elementNameNonNS);
			if (targetSchemaNode != null){
				Object substitionName = substitionElementDataStructure.get(elementName);
				TargetSchemaNode subTargetSchemaNode = (TargetSchemaNode)dataStructure.get(substitionName);
				targetSchemaNode.setSubstitutionGroup(subTargetSchemaNode);				
			}
			else{
				Object substitutionName = substitionElementDataStructure.get(elementName);
				String substitutionNameNonNS = ParserHelper.removeNameSpace((String)substitutionName);
				TargetSchemaNode subTargetSchemaNode = (TargetSchemaNode)dataStructure.get(substitutionNameNonNS);	
				if (subTargetSchemaNode != null){
					dataStructure.put(elementNameNonNS, subTargetSchemaNode);
				}
			}
		}
		
	}
	

	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
//		//no target elements defined in schema
//		if (targetRequires == null) return;
		if (ISMLConstants.SCHEMA_ELEMENT.equals(localName)){
			startSchema(uri);
		}
		else if (ISMLConstants.ELEMENT_ELEMENT.endsWith(localName)){
			String elementName = attributes.getValue(ISMLConstants.NAME_ATTRIBUTE);
			String targetElement = attributes.getValue(ISMLConstants.SML_URI, ISMLConstants.TARGETELEMENT_ATTRIBUTE);
			String targetType = attributes.getValue(ISMLConstants.SML_URI, ISMLConstants.TARGETTYPE_ATTRIBUTE);
			String targetRequired = attributes.getValue(ISMLConstants.SML_URI, ISMLConstants.TARGETREQUIRED_ATTRIBUTE);
			if ((targetElement != null) || (targetType != null) || (targetRequired != null)){ 
				TargetSchemaNode schemaNode = new TargetSchemaNode(elementName, locator.getLineNumber(), locator.getColumnNumber());;
				schemaNode.setTargetElement(targetElement);
				schemaNode.setTargetType(targetType);
				schemaNode.setRequired(targetRequired);
				dataStructure.put(elementName, schemaNode);
			}
		
		}					
	}

	public void endElement(String uri, String localName, String qName) throws SAXException {
		if (ISMLConstants.SCHEMA_ELEMENT.equals(localName)){
			endSchema(uri);
		}
	}

	public byte getPhase() {
		return ISMLConstants.GENICS_PHASE;
	}	

}
